precision lowp float;

varying lowp vec2 textureCoordinate;
varying lowp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixturePercent;
uniform lowp int saveState;

vec4 getFromColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture, p);
    }
}

vec4 getToColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture2, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture2, p);
    }
}

float check(vec2 p1, vec2 p2, vec2 p3){
    return (p1.x - p3.x) * (p2.y - p3.y) - (p2.x - p3.x) * (p1.y - p3.y);
}

bool PointInTriangle (vec2 pt, vec2 p1, vec2 p2, vec2 p3){
    bool b1, b2, b3;
    b1 = check(pt, p1, p2) < 0.0;
    b2 = check(pt, p2, p3) < 0.0;
    b3 = check(pt, p3, p1) < 0.0;
    return ((b1 == b2) && (b2 == b3));
}

bool in_left_triangle(vec2 p){
    vec2 vertex1, vertex2, vertex3;
    vertex1 = vec2(mixturePercent, 0.5);
    vertex2 = vec2(0.0, 0.5-mixturePercent);
    vertex3 = vec2(0.0, 0.5+mixturePercent);
    if (PointInTriangle(p, vertex1, vertex2, vertex3)){
        return true;
    }
    return false;
}

bool in_right_triangle(vec2 p){
    vec2 vertex1, vertex2, vertex3;
    vertex1 = vec2(1.0-mixturePercent, 0.5);
    vertex2 = vec2(1.0, 0.5-mixturePercent);
    vertex3 = vec2(1.0, 0.5+mixturePercent);
    if (PointInTriangle(p, vertex1, vertex2, vertex3)){
        return true;
    }
    return false;
}

float blur_edge(vec2 bot1, vec2 bot2, vec2 top, vec2 testPt){
    vec2 lineDir = bot1 - top;
    vec2 perpDir = vec2(lineDir.y, -lineDir.x);
    vec2 dirToPt1 = bot1 - testPt;
    float dist1 = abs(dot(normalize(perpDir), dirToPt1));

    lineDir = bot2 - top;
    perpDir = vec2(lineDir.y, -lineDir.x);
    dirToPt1 = bot2 - testPt;
    float min_dist = min(abs(dot(normalize(perpDir), dirToPt1)), dist1);

    if (min_dist < 0.005) {
        return min_dist / 0.005;
    } else  {
        return 1.0;
    };
}


vec4 transition (vec2 uv) {
    if (in_left_triangle(uv)) {
        if (mixturePercent < 0.1){
            return getFromColor(uv);
        }
        if (uv.x < 0.5){
            vec2 vertex1 = vec2(mixturePercent, 0.5);
            vec2 vertex2 = vec2(0.0, 0.5-mixturePercent);
            vec2 vertex3 = vec2(0.0, 0.5+mixturePercent);
            return mix(
                getFromColor(uv),
                getToColor(uv),
                blur_edge(vertex2, vertex3, vertex1, uv)
            );
        } else {
            if (mixturePercent > 0.0) {
                return getToColor(uv);
            } else {
                return getFromColor(uv);
            }
        }
    } else if (in_right_triangle(uv)){
        if (uv.x >= 0.5) {
            vec2 vertex1 = vec2(1.0-mixturePercent, 0.5);
            vec2 vertex2 = vec2(1.0, 0.5-mixturePercent);
            vec2 vertex3 = vec2(1.0, 0.5+mixturePercent);
            return mix(
                getFromColor(uv),
                getToColor(uv),
                blur_edge(vertex2, vertex3, vertex1, uv)
            );
        } else {
            return getFromColor(uv);
        }
    } else {
        return getFromColor(uv);
    }
}

void main(){
    gl_FragColor = transition(textureCoordinate2);
}