precision lowp float;

varying lowp vec2 textureCoordinate;
varying lowp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixturePercent;
uniform lowp int saveState;

uniform vec4 shadow_colour; // = vec4(0.,0.,0.,.6)
uniform float shadow_height; // = 0.075
uniform float bounces; // = 3.0

const float PI = 3.14159265358;

vec4 getFromColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture, p);
    }
}

vec4 getToColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture2, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture2, p);
    }
}

vec4 transition (vec2 uv) {
  float time = mixturePercent;
  float stime = sin(time * PI / 2.);
  float phase = time * PI * bounces;
  float y = (abs(cos(phase))) * (1.0 - stime);
  float d = uv.y - y;
  return mix(
    mix(
      getToColor(uv),
      shadow_colour,
      step(d, shadow_height) * (1. - mix(
        ((d / shadow_height) * shadow_colour.a) + (1.0 - shadow_colour.a),
        1.0,
        smoothstep(0.95, 1., mixturePercent) // fade-out the shadow at the end
      ))
    ),
    getFromColor(vec2(uv.x, uv.y + (1.0 - y))),
    step(d, 0.0)
  );
}


void main(){
    gl_FragColor = transition(textureCoordinate2);
}