precision lowp float;

varying lowp vec2 textureCoordinate;
varying lowp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixturePercent;
uniform lowp int saveState;

uniform float size; // = 0.04
uniform float zoom; // = 50.0
uniform float colorSeparation; // = 0.3

vec4 getFromColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture, p);
    }
}

vec4 getToColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture2, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture2, p);
    }
}

vec4 transition(vec2 p) {
  float inv = 1. - mixturePercent;
  vec2 disp = size*vec2(cos(zoom*p.x), sin(zoom*p.y));
  vec4 texTo = getToColor(p + inv*disp);
  vec4 texFrom = vec4(
    getFromColor(p + mixturePercent*disp*(1.0 - colorSeparation)).r,
    getFromColor(p + mixturePercent*disp).g,
    getFromColor(p + mixturePercent*disp*(1.0 + colorSeparation)).b,
    1.0);
  return texTo*mixturePercent + texFrom*inv;
}

void main(){
    gl_FragColor = transition(textureCoordinate2);
}