precision lowp float;

varying lowp vec2 textureCoordinate;
varying lowp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixturePercent;
uniform lowp int saveState;

#define DEG2RAD 0.03926990816987241548078304229099 // 1/180*PI

// In degrees
uniform float rotation; // = 6

// Multiplier
uniform float scale; // = 1.2

uniform float ratio;

vec4 getFromColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture, p);
    }
}

vec4 getToColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture2, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture2, p);
    }
}

vec4 transition(vec2 uv) {
  // Massage parameters
  float phase = mixturePercent < 0.5 ? mixturePercent * 2.0 : (mixturePercent - 0.5) * 2.0;
  float angleOffset = mixturePercent < 0.5 ? mix(0.0, rotation * DEG2RAD, phase) : mix(-rotation * DEG2RAD, 0.0, phase);
  float newScale = mixturePercent < 0.5 ? mix(1.0, scale, phase) : mix(scale, 1.0, phase);
  
  vec2 center = vec2(0, 0);

  // Calculate the source point
  vec2 assumedCenter = vec2(0.5, 0.5);
  vec2 p = (uv.xy - vec2(0.5, 0.5)) / newScale * vec2(ratio, 1.0);

  // This can probably be optimized (with distance())
  float angle = atan(p.y, p.x) + angleOffset;
  float dist = distance(center, p);
  p.x = cos(angle) * dist / ratio + 0.5;
  p.y = sin(angle) * dist + 0.5;
  vec4 c = mixturePercent < 0.5 ? getFromColor(p) : getToColor(p);

  // Finally, apply the color
  return c + (mixturePercent < 0.5 ? mix(0.0, 1.0, phase) : mix(1.0, 0.0, phase));
}


void main(){
    gl_FragColor = transition(textureCoordinate2);
}