precision lowp float;// Define float precision

vec4 texture(sampler2D inputTexture, vec2 texCoord) {
    return texture2D(inputTexture, texCoord);
}

/**
 * 是否是皮肤颜色
 */
bool isSkinColor(vec4 color) {
    float r = color.x * 255.0;
    float g = color.y * 255.0;
    float b = color.z * 255.0;
    return r > 95.0 && g > 40.0 && b > 20.0 && r > g && r > b && (max(r, max(g, b)) - min(r, min(g, b))) > 15.0 && (abs(r - b) > 15.0);

}

bool checkTextureCoord(vec2 coord) {
    return coord.x >= 0.0 && coord.x <= 1.0 && coord.y >= 0.0 && coord.y <= 1.0;

}

/**
  * 正态分布函数
 */
float gauthFunc(float maxValue, float centerLine, float changeRate, float x) {
    return maxValue * exp(-pow(x - centerLine, 2.0) / (2.0 * changeRate));
}


/**
 * 磨皮
 */
vec4 skinSmooth(sampler2D inputTexture, vec2 texCoord, float widthPixelStep, float heightPixelStep, float radius, float strength) {
    vec4 centerColor = texture(inputTexture, texCoord);

    if (isSkinColor(centerColor)) {
        float gauthMaxValue = 1.0;
        float gauthCenterLine = 0.0;

        vec4 colorSum = centerColor;
        float colorRateSum = 1.0;

        vec2 upVec = vec2(0.0, -heightPixelStep);
        vec2 upRightVec = vec2(widthPixelStep, -heightPixelStep);
        vec2 rightVec = vec2(widthPixelStep, 0.0);
        vec2 rightDownVec = vec2(widthPixelStep, heightPixelStep);
        vec2 downVec = vec2(0.0, heightPixelStep);
        vec2 downLeftVec = vec2(-widthPixelStep, heightPixelStep);
        vec2 leftVec = vec2(-widthPixelStep, 0.0);
        vec2 leftUpVec = vec2(-widthPixelStep, -heightPixelStep);
        for (float i = 1.0; i <= radius; i = i + 1.0) {
            float colorRate = gauthFunc(gauthMaxValue, gauthCenterLine, strength, i);
            vec2 u = texCoord + upVec * i;
            if (checkTextureCoord(u)) {
                vec4 c = texture(inputTexture, u);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 ur = texCoord + upRightVec * i;
            if (checkTextureCoord(ur)) {
                vec4 c = texture(inputTexture, ur);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 r = texCoord + rightVec * i;
            if (checkTextureCoord(r)) {
                vec4 c = texture(inputTexture, r);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 rd = texCoord + rightDownVec * i;
            if (checkTextureCoord(rd)) {
                vec4 c = texture(inputTexture, rd);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 d = texCoord + downVec * i;
            if (checkTextureCoord(d)) {
                vec4 c = texture(inputTexture, d);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 dl = texCoord + downLeftVec * i;
            if (checkTextureCoord(dl)) {
                vec4 c = texture(inputTexture, dl);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 l = texCoord + leftVec * i;
            if (checkTextureCoord(l)) {
                vec4 c = texture(inputTexture, l);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }

            vec2 lu = texCoord + leftUpVec * i;
            if (checkTextureCoord(lu)) {
                vec4 c = texture(inputTexture, lu);
                if (isSkinColor(c)) {
                    colorRateSum += colorRate;
                    colorSum += colorRate * c;
                }
            }
        }
        return colorSum / colorRateSum;
    } else {
        return centerColor;
    }
}

uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;
uniform float mixturePercent;

// 磨皮
uniform int skinSmoothSwitch;
uniform float textureWidthPixelStep;
uniform float textureHeightPixelStep;
uniform float skinSmoothStrength;
//Mix 0.6
uniform float skinSmoothMix;
//radius 6.0
uniform float skinSmoothRadius;

void main() {
    vec4 outputColor = texture(inputImageTexture, textureCoordinate);

    // 磨皮
    if (skinSmoothSwitch == 1) {
        vec4 smoothColor = skinSmooth(inputImageTexture, textureCoordinate, textureWidthPixelStep, textureHeightPixelStep, skinSmoothRadius, skinSmoothStrength);
        outputColor = mix(outputColor, smoothColor, skinSmoothMix);
    }
    gl_FragColor = outputColor;
}