precision lowp float;

varying lowp vec2 textureCoordinate;
varying lowp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixturePercent;
uniform lowp int saveState;

uniform float intensity; // = 0.1
const int passes = 6;

vec4 getFromColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture, p);
    }
}

vec4 getToColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture2, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture2, p);
    }
}

vec4 transition(vec2 uv) {
    vec4 c1 = vec4(0.0);
    vec4 c2 = vec4(0.0);

    float disp = intensity*(0.5-distance(0.5, mixturePercent));
    for (int xi=0; xi<passes; xi++)
    {
        float x = float(xi) / float(passes) - 0.5;
        for (int yi=0; yi<passes; yi++)
        {
            float y = float(yi) / float(passes) - 0.5;
            vec2 v = vec2(x,y);
            float d = disp;
            c1 += getFromColor( uv + d*v);
            c2 += getToColor( uv + d*v);
        }
    }
    c1 /= float(passes*passes);
    c2 /= float(passes*passes);
    return mix(c1, c2, mixturePercent);
}

void main(){
    gl_FragColor = transition(textureCoordinate2);
}