precision mediump float;

varying lowp vec2 textureCoordinate;
varying lowp vec2 textureCoordinate2;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float mixturePercent;
uniform lowp int saveState;

uniform float amplitude; // = 1.0
uniform float waves; // = 30.0
uniform float colorSeparation; // = 0.3

const float PI = 3.14159265358979323846264;

vec4 getFromColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture, p);
    }
}

vec4 getToColor(vec2 p){
    if(saveState==1){
        return texture2D(inputImageTexture2, vec2(p.x,1.0-p.y));
    } else {
        return texture2D(inputImageTexture2, p);
    }
}

float compute(vec2 p, float mixturePercent, vec2 center) {
vec2 o = p*sin(mixturePercent * amplitude)-center;
// horizontal vector
vec2 h = vec2(1., 0.);
// butterfly polar function (don't ask me why this one :))
float theta = acos(dot(o, h)) * waves;
return (exp(cos(theta)) - 2.*cos(4.*theta) + pow(sin((2.*theta - PI) / 24.), 5.)) / 10.;
}
vec4 transition(vec2 uv) {
  vec2 p = uv.xy / vec2(1.0).xy;
  float inv = 1. - mixturePercent;
  vec2 dir = p - vec2(.5);
  float dist = length(dir);
  float disp = compute(p, mixturePercent, vec2(0.5, 0.5)) ;
  vec4 texTo = getToColor(p + inv*disp);
  vec4 texFrom = vec4(
  getFromColor(p + mixturePercent*disp*(1.0 - colorSeparation)).r,
  getFromColor(p + mixturePercent*disp).g,
  getFromColor(p + mixturePercent*disp*(1.0 + colorSeparation)).b,
  1.0);
  return texTo*mixturePercent + texFrom*inv;
}

void main(){
    gl_FragColor = transition(textureCoordinate2);
}